#!/usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _compiler = require('./compiler');

var _compiler2 = _interopRequireDefault(_compiler);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

var _package = require('../../package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var input = void 0;

_commander2['default'].version(_package.version).arguments('<file_or_directory>').option('-o, --output <file_or_directory>', 'destination file or folder for compiled files').option('-x, --extensions <extensions>', 'list of extensions to hook into', function (val) {
  return val.split(',').map(function (x) {
    return String(x).replace(/(?:(^")|("$))/g, '');
  });
}).option('-i, --ignore <regex>', 'ignore all files and directories that match this regex').option('--no-gccxrc', 'whether or not to look up .gccxrc').option('--no-copy-files', 'when compiling a directory avoid copy over non-compilable files').option('-w, --watch', 'compile files every time that you change them').action(function (fileOrDirectory) {
  input = fileOrDirectory;
}).parse(process.argv);

if (input === undefined) {
  _commander2['default'].outputHelp();
  (0, _utils.exit)({
    code: 1
  });
}

var config = (0, _config2['default'])(input, _commander2['default']);

// eslint-disable-next-line
console.log('\n');

var compileFirst = function compileFirst() {
  return (0, _compiler2['default'])(config.input, config.output, _commander2['default'], true);
};

if (config.watch) {
  var getOutputPath = function getOutputPath(inputPath) {
    var result = void 0;
    if (config.output) {
      var relative = config.input !== inputPath ? _path2['default'].relative(config.input, inputPath) : inputPath;
      result = _path2['default'].resolve(config.output, relative);
      if (result === relative) {
        result = config.output;
      }
    }
    return result;
  };

  var onCompilationError = function onCompilationError(file, ex) {
    if (ex !== undefined && ex !== null) {
      // eslint-disable-next-line
      console.error('\nError in file: ' + file + '\n\n' + (ex.message || ex) + '\n');
    }
  };

  compileFirst()['catch'](onCompilationError.bind(null, config.input));

  var compilePath = function compilePath(changed) {
    (0, _compiler2['default'])(changed, getOutputPath(changed), _commander2['default'])['catch'](onCompilationError.bind(null, changed));
  };

  _chokidar2['default'].watch(config.input, {
    persistent: true,
    ignoreInitial: true
  }).on('change', compilePath).on('add', compilePath).on('addDir', function (created) {
    var output = getOutputPath(created);
    if (!_fs2['default'].existsSync(output)) {
      _fs2['default'].mkdirSync(output);
    }
  }).on('error', function (error) {
    return (0, _utils.exit)({
      code: 1,
      message: error
    });
  });
} else {
  compileFirst()['catch'](function (ex) {
    (0, _utils.exit)({
      code: 1,
      message: ex && ex.message ? ex.message : ex
    });
  });
}